unit fMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, cHTTP, cTCPClient, cTCPStream, cHTTPClient, cHTTPMessage,
  cSocketHostLookup, cWindows;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    ePath: TEdit;
    Button1: TButton;
    Memo1: TMemo;
    Label3: TLabel;
    ePort: TEdit;
    Button2: TButton;
    eHost: TComboBox;
    HTTPClient: TelHTTPClient;
    HTTPRequest: TelHTTPRequest;
    procedure Button1Click(Sender: TObject);
    procedure fndTimerHandle1Timer(const Sender: TTimerHandle);
    procedure HTTPClientResponse(const Sender: ThttpClient);
    procedure HTTPClientSyncClose(const Sender: ATCPClient);
    procedure HTTPClientSyncConnected(const Sender: ATCPClient);
    procedure HTTPClientSyncConnectFailed(const Sender: ATCPClient);
    procedure HTTPClientSyncResponse(const Sender: ThttpClient);
    procedure HTTPClientSyncResponseLine(const Sender: ThttpClient);
    procedure HTTPClientSyncResponseHeader(const Sender: ThttpClient);
    procedure HTTPClientSyncHeaderSent(const Sender: ThttpClient);
    procedure Button2Click(Sender: TObject);
    procedure HTTPClientSyncContentBlock(const Sender: ThttpClient;
      const Buffer: Pointer; const Size: Integer; var Abort: Boolean);
  private
    ContentStr : String;
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.Button1Click(Sender: TObject);
begin
  try
    if HTTPClient.Active then
      HTTPClient.Active := False;

    HTTPRequest.Method := http_GET;
    HTTPRequest.URI := ePath.Text;

    HTTPClient.Host := eHost.Text;
    HTTPClient.Port := ePort.Text;

    HTTPClient.Active := True;
  finally
  end;
end;


procedure TForm1.Button2Click(Sender: TObject);
begin
  HTTPClient.Active := False;
end;

procedure TForm1.fndTimerHandle1Timer(const Sender: TTimerHandle);
begin
  Memo1.Lines.Add ('Timer');
end;

// HTTPClientResponse is linked to the "unsynced" event (ie it is called
// in the client thread). OnSyncResponse is called in the main thread.
procedure TForm1.HTTPClientResponse(const Sender: ThttpClient);
begin
  // If HTTPClient.ContentReadMode = hcGetContentStream then this function
  // is called as soon as the content starts;
  // If HTTPClient.ContentReadMode = hcGetContentString then this function
  // is called after the complete content is available as a string

  // Partial content can be read using Sender.Response.ContentAsStream.Read()
  // Synchronizing with the main thread can be achieved using Sender.Synchronize

  // This function simply saves the complete content in a string
  if Sender.ContentReadMode <> hcSaveContentFile then
    ContentStr := Sender.Response.ContentAsString;
end;

// All the "Sync" events are called in the main thread

procedure TForm1.HTTPClientSyncClose(const Sender: ATCPClient);
begin
  Memo1.Lines.Add ('Close');
end;

procedure TForm1.HTTPClientSyncConnected(const Sender: ATCPClient);
begin
  Memo1.Lines.Add ('Connected');
end;

procedure TForm1.HTTPClientSyncConnectFailed(const Sender: ATCPClient);
begin
  Memo1.Lines.Add ('Connect failed');
end;

procedure TForm1.HTTPClientSyncResponse(const Sender: ThttpClient);
begin
  Memo1.Lines.Add ('Content');
  Memo1.Lines.Add (ContentStr);
  Memo1.Lines.Add ('Content Fin');
end;

procedure TForm1.HTTPClientSyncResponseLine(const Sender: ThttpClient);
begin
  Memo1.Lines.Add ('Response Line');
  Memo1.Lines.Add (Sender.Response.StartLine);
end;

procedure TForm1.HTTPClientSyncResponseHeader(
  const Sender: ThttpClient);
begin
  Memo1.Lines.Add ('Response Header');
  Memo1.Lines.Add (Sender.Response.HeaderAsString);
end;

procedure TForm1.HTTPClientSyncHeaderSent(const Sender: ThttpClient);
begin
  Memo1.Lines.Add ('Header Sent');
  Memo1.Lines.Add (Sender.Request.HeaderAsString);
end;

procedure TForm1.HTTPClientSyncContentBlock(const Sender: ThttpClient;
  const Buffer: Pointer; const Size: Integer; var Abort: Boolean);
var S : String;
begin
  // This event is triggered if HTTPClient.ContentReadMode = hcNotifyContentBlocks
  // It is called for every block of data received. Set Abort to True to
  // close the connection.
  if Size <= 0 then
    exit;
  SetLength (S, Size);
  Move (Buffer^, Pointer (S)^, Size);
  Memo1.Lines.Add ('Content block');
  Memo1.Lines.Add (S);
end;

end.
